Package: lefser
Type: Package
Title: R implementation of the LEfSE method for microbiome biomarker
        discovery
Description: lefser is the R implementation of the popular microbiome 
  biomarker discovery too, LEfSe. It uses the Kruskal-Wallis test, 
  Wilcoxon-Rank Sum test, and Linear Discriminant Analysis to find biomarkers 
  from two-level classes (and optional sub-classes).
Version: 1.20.0
Date: 2025-04-22
Authors@R: c(
  person("Sehyun", "Oh", , "shbrief@gmail.com", c("cre", "ctb"), c(ORCID = "0000-0002-9490-3061")),
  person("Asya", "Khleborodova", , "asya.bioconductor@gmail.com", "aut"),
  person("Samuel", "Gamboa-Tuz", , "Samuel.Gamboa.Tuz@gmail.com", "ctb"),
  person("Marcel", "Ramos", , "marcel.ramos@sph.cuny.edu", "ctb", c(ORCID = "0000-0002-3242-0582")),
  person("Ludwig", "Geistlinger", , "Ludwig.Geistlinger@sph.cuny.edu", "ctb", c(ORCID = "0000-0002-2495-5464")),
  person("Levi", "Waldron", , "levi.waldron@sph.cuny.edu", "ctb", c(ORCID = "0000-0003-2725-0694"))
  )
License: Artistic-2.0
Depends: SummarizedExperiment, R (>= 4.5.0)
Imports: coin, MASS, ggplot2, S4Vectors, stats, methods, utils, dplyr,
        testthat, tibble, tidyr, forcats, stringr, ggtree,
        BiocGenerics, ape, ggrepel, mia, purrr, tidyselect, treeio
Suggests: knitr, rmarkdown, curatedMetagenomicData, BiocStyle,
        phyloseq, pkgdown, covr, withr
Encoding: UTF-8
BugReports: https://github.com/waldronlab/lefser/issues
URL: https://github.com/waldronlab/lefser
VignetteBuilder: knitr
biocViews: Software, Sequencing, DifferentialExpression, Microbiome,
        StatisticalMethod, Classification
RoxygenNote: 7.3.2
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/lefser
git_branch: RELEASE_3_22
git_last_commit: c184c36
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 22:22:42 UTC; biocbuild
Author: Sehyun Oh [cre, ctb] (ORCID: <https://orcid.org/0000-0002-9490-3061>),
  Asya Khleborodova [aut],
  Samuel Gamboa-Tuz [ctb],
  Marcel Ramos [ctb] (ORCID: <https://orcid.org/0000-0002-3242-0582>),
  Ludwig Geistlinger [ctb] (ORCID:
    <https://orcid.org/0000-0002-2495-5464>),
  Levi Waldron [ctb] (ORCID: <https://orcid.org/0000-0003-2725-0694>)
Maintainer: Sehyun Oh <shbrief@gmail.com>

# Generated by roxygen2: do not edit by hand

export(get_terminal_nodes)
export(lefser)
export(lefserClades)
export(lefserPlot)
export(lefserPlotClad)
export(lefserPlotFeat)
export(relativeAb)
export(rowNames2RowData)
import(SummarizedExperiment)
import(ggplot2)
importFrom(MASS,lda)
importFrom(coin,pvalue)
importFrom(coin,statistic)
importFrom(coin,wilcox_test)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,mutate)
importFrom(dplyr,row_number)
importFrom(ggtree,"%<+%")
importFrom(methods,as)
importFrom(methods,is)
importFrom(stats,kruskal.test)
importFrom(stats,reorder)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(testthat,capture_warnings)
importFrom(utils,head)
importFrom(utils,tail)

<!-- badges: start -->
![build](https://github.com/waldronlab/lefser/workflows/build/badge.svg)
[![Codecov test coverage](https://codecov.io/gh/waldronlab/lefser/branch/devel/graph/badge.svg)](https://codecov.io/gh/waldronlab/lefser?branch=devel)
<!-- badges: end -->

## *lefser*: Run *LEfSe* in R
*lefser* is the R implementation of the Python package, Linear discriminant 
analysis (LDA) Effect Size (*[LEfSe][]*). *LEfSe* is the most widely used Python 
package and Galaxy module for metagenomic biomarker discovery and 
visualization ([Segata et al. 2011][]). *LEfSe* utilizes standard 
statistical significance tests along with supplementary tests that incorporate 
biological consistency and the relevance of effects to identity the features 
(e.g., organisms, clades, OTU, genes, or functions) that are most likely to 
account for differences between the two sample classes of interest, referred as
‘classes’. While *LEfSe* is widely used and available in different platform 
such as Galaxy UI and Conda, there is no convenient way to incorporate it in 
R-based workflows. Thus, we re-implement *LEfSe* as an R/Bioconductor package, 
*lefser*. Following the *LEfSe*‘s algorithm including Kruskal-Wallis test, 
Wilcoxon-Rank Sum test, and Linear Discriminant Analysis, with some 
modifications, *lefser* successfully reproduces and improves the original 
statistical method and the associated plotting functionality.

[LEfSe]: https://huttenhower.sph.harvard.edu/galaxy/
[Segata et al. 2011]: https://www.ncbi.nlm.nih.gov/pmc/articles/PMC3218848/

------------------------------------------------------------------------

#### Citation

If you use *lefser* in published research, please cite:

> Asya Khleborodova, Samuel D Gamboa-Tuz, Marcel Ramos, Nicola Segata, Levi Waldron, Sehyun Oh,
> *Lefser*: Implementation of metagenomic biomarker discovery tool, *LEfSe*, in R, *Bioinformatics*, 2024;, btae707, [doi:10.1093/bioinformatics/btae707](https://doi.org/10.1093/bioinformatics/btae707)


A BibTeX entry for LaTeX users is: 

     @article{10.1093/bioinformatics/btae707,
      author = {Khleborodova, Asya and Gamboa-Tuz, Samuel D and Ramos, Marcel and Segata, Nicola and Waldron, Levi and Oh, Sehyun},
      title = {Lefser: Implementation of metagenomic biomarker discovery tool, LEfSe, in R},
      journal = {Bioinformatics},
      pages = {btae707},
      year = {2024},
      month = {11},
      issn = {1367-4811},
      doi = {10.1093/bioinformatics/btae707},
      url = {https://doi.org/10.1093/bioinformatics/btae707},
      eprint = {https://academic.oup.com/bioinformatics/advance-article-pdf/doi/10.1093/bioinformatics/btae707/60811200/btae707.pdf},
    }

wilcox_pstats <- function(datamat, class, index) {
  apply(datamat, 1L, function(values) {
    wx <- suppressWarnings({
        coin::wilcox_test(values ~ class, subset = index)
    })
    c(pvalue = coin::pvalue(wx), statistic = coin::statistic(wx))
  })
}

## Wilcoxon Rank-Sum Test for the sub-classes
fillPmatZmat <- function(class,
                         subclass,
                         relab_sub,
                         p.threshold,
                         method)
{
  if (!nrow(relab_sub))
    return(relab_sub)
  # creates a list of boolean vectors, each vector indicates
  # existence (TRUE) or absence (FALSE) of a class/sub-class combination
  combos <- apply(
    expand.grid(levels(class), levels(subclass)), 1L, paste0, collapse = "."
  )
  combined <- interaction(class, subclass)
  whichlist <- lapply(
    setNames(nm = sort(combos)), function(combo) which(combo == combined)
  )

  ## uses Wilcoxon rank-sum test to test for significant differential abundances between
  ## subclasses of one class against subclasses of all other classes; results are saved in
  ## "pval_mat" and "z_mat" matrices
  ssubclass <- seq_along(levels(subclass))
  iters <- expand.grid(ssubclass, ssubclass + length(ssubclass))
  inds <- apply(iters, 1L, function(x) unlist(whichlist[x], use.names = FALSE))

  z_mat <- pval_mat <- matrix(
    NA, nrow = nrow(relab_sub), ncol = length(inds),
    dimnames = list(rownames(relab_sub), NULL)
  )
  for (i in seq_along(inds)) {
    results <- wilcox_pstats(relab_sub, class = class, index = inds[[i]])
    pval_mat[, i] <- stats::p.adjust(results["pvalue", ], method = method)
    z_mat[, i] <- results["statistic", ]
  }

  ## converts "pval_mat" into boolean matrix "logical_pval_mat" where
  ## p-values <= wilcoxon.threshold
  logical_pval_mat <- !is.na(pval_mat) & pval_mat <= p.threshold * 2.0

  ## determines which rows (features) have all p-values<=0.05
  ## and selects such rows from the matrix of z-statistics
  sub <- rowSums(logical_pval_mat) == ncol(logical_pval_mat)
  z_mat_sub <- z_mat[sub, , drop = FALSE]
  ## confirms that z-statistics of a row all have the same sign
  sub <- abs(rowSums(z_mat_sub)) == rowSums(abs(z_mat_sub))
  relab_sub[names(sub[sub]), , drop = FALSE]
}

## ensures that more than half of the values in each for each feature are unique
## if that is not the case then a count value is altered by adding it to a small value
## generated via normal distribution with mean=0 and sd=5% of the count value
createUniqueValues <- function(df, class){
  orderedrows <- rownames(df)
  splitdf <- split(df, class)
  maxim <- vapply(table(class), function(x) max(x * 0.5, 4), numeric(1L))
  for (i in seq_along(splitdf)) {
    sdat <- splitdf[[i]]
    splitdf[[i]][] <- lapply(sdat, function(cols) {
        if (length(unique(cols)) > maxim[i])
            cols
        else
            abs(cols + rnorm(
                length(cols), mean = 0, sd = max(cols * 0.05, 0.01))
            )
    })
  }
  df <- do.call(rbind, unname(splitdf))
  df[match(orderedrows, rownames(df)),, drop = FALSE]
}


# Perform LDA modeling
#
# @param data A data frame with z-score values. Rows are samples and columns
# are features that pass the significance cutoff.
# @param classes The names of classes for the main class.
#
ldaFunction <- function (data, classes) {

    ## Fitting LDA model
    lda.fit <- lda(class ~ ., data = data)
    w <- lda.fit$scaling[, 1] # extract LDA coefficients
    w.unit <- w / sqrt(sum(w ^ 2)) # scaling LDA coefficient by their Euclidean norm to get unit-normalized coefficient

    ss <- data[,-match("class", colnames(data)), drop = FALSE]
    xy.matrix <- as.matrix(ss) # the original feature matrix

    ## Transform the original feature matrix
    LD <- xy.matrix %*% w.unit # discriminant scores

    ## Calculating Effect Size for each feature
    ## Effect Size = the absolute difference between the mean discriminant scores of the two classes
    effect_size <-
        abs(mean(LD[data[, "class"] == 1]) - mean(LD[data[, "class"] == 0]))

    ## Coefficient scaling
    ## Scale the unit-normalized LDA coefficients by the effect size
    scal <- w.unit * effect_size # scaled LDA coefficient

    coeff <- vector("numeric", length(scal)) # absolute scaled LDA coefficient
    for (v in seq_along(scal)) {
        if (!is.na(scal[v])) {
            coeff[v] <- abs(scal[v])
        } else{
            coeff[v] <- 0
        }
    }

    ## Feature Importance
    lda.means.diff <- (lda.fit$means[2,] - lda.fit$means[1,]) # difference between the class means for each feature
    res <- (lda.means.diff + coeff) / 2
    names(res) <- colnames(lda.fit$means)

    return(res)
}


## Kruskal-Wallis Rank Sum Test for the classes
filterKruskal <- function(relab, class, p.value, method = method) {
  # applies "kruskal.test.alt" function to each row (feature) of relab
  # to detect differential abundance between classes, 0 and 1
  kw.res <- apply(relab, 1L, function(x) {
    kruskal.test(x ~ class)[["p.value"]]
  })
  # TRUE for p-values less than or equal to kw.threshold
  kw.res <- stats::p.adjust(kw.res, method = method)
  kw.sub <- kw.res < p.value

  # NAs are FALSE
  kw.sub[is.na(kw.sub)] <- FALSE

  # extracts features with statistically significant differential abundance
  # from "relab" matrix
  relab[kw.sub,, drop = FALSE]
}

#' R implementation of the LEfSe method
#'
#' Perform a LEfSe analysis: the function carries out differential analysis
#' between two sample classes for multiple features and uses linear discriminant analysis
#' to establish their effect sizes. Subclass information for each class can be incorporated
#' into the analysis (see examples). Features with large differences between two sample classes
#' are identified as biomarkers.
#'
#' @details
#' The LEfSe method expects relative abundances in the `expr` input. A warning
#' will be emitted if the column sums do not result in 1. Use the \code{relativeAb}
#' helper function to convert the data in the
#' [`SummarizedExperiment`][SummarizedExperiment::SummarizedExperiment-class]
#' to relative abundances. The `checkAbundances` argument enables checking the
#' data for presence of relative abundances and can be turned off by setting
#' the argument to `FALSE`.
#'
#' @param relab A [SummarizedExperiment-class][SummarizedExperiment::SummarizedExperiment-class]
#' with relative abundances in the assay
#' @param kruskal.threshold numeric(1) The p-value for the Kruskal-Wallis Rank
#' Sum Test (default 0.05). If multiple hypothesis testing is performed, this
#' threshold is applied to corrected p-values.
#' @param wilcox.threshold numeric(1) The p-value for the Wilcoxon Rank-Sum Test
#' when 'subclassCol' is present (default 0.05). If multiple hypothesis testing is
#' performed, this threshold is applied to corrected p-values.
#' @param lda.threshold numeric(1) The effect size threshold (default 2.0).
#' @param classCol character(1) Column name in `colData(relab)` indicating
#' class, usually a factor with two levels (e.g., `c("cases", "controls")`;
#' default "CLASS").
#' @param subclassCol character(1) Optional column name in `colData(relab)`
#' indicating the subclasses, usually a factor with two levels (e.g.,
#' `c("adult", "senior")`; default NULL), but can be more than two levels.
#' @param assay The i-th assay matrix in the
#' [`SummarizedExperiment`][SummarizedExperiment::SummarizedExperiment-class]
#' ('relab'; #' default 1).
#' @param trim.names Default is `FALSE`. If `TRUE`, this function extracts
#' the most specific taxonomic rank of organism.
#' @param checkAbundances `logical(1)` Whether to check if the assay data in the
#'   `relab` input are relative abundances or counts. If counts are found, a
#'   warning will be emitted (default `TRUE`).
#' @param method Default is "none" as in the original LEfSe implementation.
#' Character string of length one, passed on to \link[stats]{p.adjust} to set
#' option for multiple testing. For multiple pairwise comparisons, each comparison
#' is adjusted separately. Options are "holm", "hochberg", "hommel",
#' "bonferroni", "BH", "BY", "fdr" (synonym for "BH"), and "none".
#' @param \ldots Additional inputs to lower level functions (not used).
#' @return
#' The function returns a `data.frame` with two columns, which are
#' names of features and their LDA scores.
#'
#' @importFrom stats kruskal.test reorder rnorm
#' @importFrom coin pvalue statistic wilcox_test
#' @importFrom MASS lda
#' @importFrom methods as is
#' @importFrom stats setNames
#' @importFrom utils tail
#' @importFrom testthat capture_warnings
#' @import SummarizedExperiment
#'
#' @examples
#'
#'     data(zeller14)
#'     zeller14 <- zeller14[, zeller14$study_condition != "adenoma"]
#'     tn <- get_terminal_nodes(rownames(zeller14))
#'     zeller14tn <- zeller14[tn,]
#'     zeller14tn_ra <- relativeAb(zeller14tn)
#'
#'     # (1) Using classes only
#'     res_class <- lefser(zeller14tn_ra,
#'                         classCol = "study_condition")
#'     # (2) Using classes and sub-classes
#'     res_subclass <- lefser(zeller14tn_ra,
#'                         classCol = "study_condition",
#'                         subclassCol = "age_category")
#'
#' @export
lefser <-
  function(relab,
           kruskal.threshold = 0.05,
           wilcox.threshold = 0.05,
           lda.threshold = 2.0,
           classCol = "CLASS",
           subclassCol = NULL,
           assay = 1L,
           trim.names = FALSE,
           checkAbundances = TRUE,
           method = "none",
           ...
) {
    relab_data <- assay(relab, i = assay)

    ## Check whether relative abundance is provided or not
    if (checkAbundances && !identical(all.equal(colSums(relab_data),
                                                rep(1e6, ncol(relab_data)),
                                                check.attributes = FALSE),
                                      TRUE)) {
        warning("Convert counts to relative abundances with 'relativeAb()'")
    }

    ## Extract the class/subclass information
    classf <- colData(relab)[[classCol]]
    classf <- as.factor(classf)
    lclassf <- levels(classf)
    if (is.null(classf) || !identical(length(lclassf), 2L)) {
        msg <- "'classCol' must refer to a valid dichotomous (two-level) variable"
        stop(msg) # ensure the class has only two levels
    }
    message(
        "The outcome variable is specified as '", classCol,
        "' and the reference category is '", lclassf[1],
        "'.\n See `?factor` or `?relevel` to change the reference category."
    )

    ## Kruskal-Wallis Rank Sum Test for the classes
    relab_sub <- filterKruskal(relab = relab_data,
                               class = classf,
                               p.value = kruskal.threshold,
                               method = method)

    ## Wilcoxon Rank-Sum Test for the sub-classes
    if (!is.null(subclassCol)) {
        subclass <- as.factor(colData(relab)[[subclassCol]])
        subclass <- droplevels(subclass)
        ## z-statistics result of the features passing the significance cut-off
        relab_sub <- fillPmatZmat(class = classf,
                                  subclass = subclass,
                                  relab_sub = relab_sub,
                                  p.threshold = wilcox.threshold,
                                  method = method)
    }

    ## Return an empty data table if there is no significant features
    if(nrow(relab_sub) == 0L){
      return(.return_no_results())
    }

    ## Transposed relative abundance matrix with the 'class' column
    relab_sub_t <- t(relab_sub)
    relab_sub_t_df <- as.data.frame(relab_sub_t)

    # relab_sub_t_df <- createUniqueValues(df = relab_sub_t_df, class = classf)
    relab_sub_t_df <- cbind(relab_sub_t_df, class = classf)

    ## LDA model
    warn <- testthat::capture_warnings(
        raw_lda_scores <- ldaFunction(relab_sub_t_df, lclassf)
    )

    ## Warning collinearity and recommend `get_terminal_nodes`
    if (length(warn) && nzchar(warn)) {
        msg <- "Variables in the input are collinear. Try only with the terminal nodes using `get_terminal_nodes` function"
        warning(msg)
    }
    ## Processing LDA scores
    processed_scores <-
      sign(raw_lda_scores) * log((1 + abs(raw_lda_scores)), 10)
    processed_sorted_scores <- sort(processed_scores)   # sorting of scores
    scores_df <- data.frame(features = names(processed_sorted_scores),
                            scores = as.vector(processed_sorted_scores),
                            stringsAsFactors = FALSE)
    scores_df <- .trunc(scores_df, trim.names)   # short-form of taxa name

    ## Filter with LDA threshold
    threshold_scores <- abs(scores_df$scores) >= lda.threshold
    res_scores <- scores_df[threshold_scores, , drop = FALSE]
    class(res_scores) <- c("lefser_df", class(res_scores))
    attr(res_scores, "classes") <- lclassf
    if (nrow(res_scores) == 0L) {
        return(.return_no_results())
    }

    ## Add attributes with argument values
    ## This is used for plottin functions
    attr(res_scores, "inputSE") <- relab
    attr(res_scores, "kth") <-  kruskal.threshold
    attr(res_scores, "wth") <- wilcox.threshold
    attr(res_scores, "ldath") <- lda.threshold
    attr(res_scores, "class_arg") <- classCol
    attr(res_scores, "subclass_arg") <- subclassCol
    attr(res_scores, "method") <- method
    # attr(res_scores, "lgroupf") <- lgroupf[1]
    # attr(res_scores, "case") <- lgroupf[2]

    ## Some more attributes to create the cladogram.
    # pathStrings <- .selectPathStrings(relab, res_scores)
    # # attr(res_scores, "pathStrings") <- pathStrings
    # attr(res_scores, "tree") <- .toTree(pathStrings)
    attr(res_scores, "lclassf") <- lclassf[1]
    attr(res_scores, "case") <- lclassf[2]

    res_scores
 }

.return_no_results <- function() {
    message("No significant features found.")
    res_scores <- data.frame(features = character(), scores = numeric())
    class(res_scores) <- c("lefser_df", class(res_scores))
    return(res_scores)
}

#' Plot Feature
#'
#' \code{lefserPlotFeat} plots the abundance data of a DA feature across all
#' samples.
#'
#' @param res An object of class lefser_df,
#' output of the \code{lefser} function.
#' @param fName A character string. The name of a feature in the lefser_df
#' object.
#' @param colors Colors corresponding to class 0 and 1.
#' Options: "c" (colorblind), "l" (lefse), "g" (greyscale). Defaults to "c".
#' This argument also accepts a character(2) with two color names.
#'
#' @details
#' The solid lines represent the mean by class or by class+subclass
#' (if the subclass variable is present).
#' The dashed lines represent the median by class or by class+subclass
#' (if the subclass variable is present).
#'
#' @return A ggplot object.
#' @export
#'
#' @examples
#'
#' data(zeller14)
#' zeller14 <- zeller14[, zeller14$study_condition != "adenoma"]
#' tn <- get_terminal_nodes(rownames(zeller14))
#' zeller14tn <- zeller14[tn,]
#' zeller14tn_ra <- relativeAb(zeller14tn)
#'
#' # (1) Using classes only
#' res_class <- lefser(zeller14tn_ra,
#'                     classCol = "study_condition")
#' # (2) Using classes and sub-classes
#' res_subclass <- lefser(zeller14tn_ra,
#'                     classCol = "study_condition",
#'                     subclassCol = "age_category")
#' plot_class <- lefserPlotFeat(res_class, res_class$features[[1]])
#' plot_subclass <- lefserPlotFeat(res_subclass, res_subclass$features[[2]])
#'
lefserPlotFeat <- function(res, fName, colors = "colorblind") {
    dat <- .prepareDataHistogram(res = res, fName = fName)
    refclass <- attr(res, "lclassf")
    vLinePos <- which(dat$classCol != refclass)[1] - 0.5
    colVar <- .selectPalette(colors)
    l <- split(dat, dat$classCol)
    # maxYVal <- max(dat$abundance)
    maxYVal <- 1
    vals <- purrr::map(l, ~ {
        head(.x[["sample"]], 1)
    })
    sumDat <- .sumDatHist(dat)
    cond <- "subclassCol" %in% colnames(dat)
    if (isFALSE(cond)) {
        p <- dat |>
            ggplot2::ggplot(
                data = dat, mapping = ggplot2::aes(.data$sample, .data$abundance)
            ) +
            ggplot2::geom_col(
                mapping = ggplot2::aes(fill = .data$classCol), width = 1
            ) +
            ggplot2::scale_fill_manual(
                values = colVar
            )
    } else if (isTRUE(cond)) {
        p <- dat |>
            ggplot2::ggplot(
                data = dat,
                mapping = ggplot2::aes(.data$sample, .data$abundance)
            ) +
            ggplot2::geom_col(
                mapping = ggplot2::aes(fill = .data$subclassCol), width = 1
            ) +
            ggplot2::scale_fill_manual(
                values = colVar
            )
    }
    p <- p +
        ggplot2::scale_y_continuous(
            limits = c(0, 1),
            # labels = scales::scientific,
            expand = c(0, 0)
        ) +
        ggplot2::labs(
            x = "Samples", y = "Relative abundance",
            title = stringr::str_wrap(fName, whitespace_only = FALSE)
        ) +
        ggplot2::annotate(
            geom = "label",
            x = vals[[levels(dat$classCol)[1]]],
            y = maxYVal,
            label = levels(dat$classCol)[1],
            hjust = 0, vjust = 1
        ) +
        ggplot2::annotate(
            geom = "label",
            x = vals[[levels(dat$classCol)[2]]],
            y = maxYVal,
            label = levels(dat$classCol)[2],
            hjust = 0, vjust = 1
        ) +
        ggplot2::geom_vline(xintercept = vLinePos) +
        theme_bw() +
        theme(
            axis.text.x = ggplot2::element_blank(),
            axis.ticks.x = ggplot2::element_blank(),
            panel.grid.major.x = ggplot2::element_blank(),
            legend.title = ggplot2::element_blank()
        )
    for (i in seq_along(sumDat)) {
        p <- p +
            ggplot2::geom_segment(
                data = sumDat[[i]],
                mapping = ggplot2::aes(
                    x = .data$x1, xend = .data$x2,
                    y = .data$mean1, yend = .data$mean2
                ),
                linetype = 1
            ) +
            ggplot2::geom_segment(
                data = sumDat[[i]],
                mapping = ggplot2::aes(
                    x = .data$x1, xend = .data$x2,
                    y = .data$median1, yend = .data$median2
                ),
                linetype = 2
            )
    }
    return(p)
}

.sumDatHist <- function(x) {
    cond <- "subclassCol" %in% colnames(x)
    if (isFALSE(cond)) {
        l <- split(x, x$classCol)
    } else if (isTRUE(cond)) {
       l <- x |>
           dplyr::mutate(
               classes = paste0(.data$classCol, "--", .data$subclassCol)
           ) |>
           {\(y) split(y, y[["classes"]])}()
    }
    sumDat <- purrr::map(l, ~ {
        meanVal <- mean(.x[["abundance"]] )
        medianVal <- median(.x[["abundance"]] )
        minVal <- head(.x[["sample"]], 1)
        maxVal <- tail(.x[["sample"]], 1)
        data.frame(
            x1 = minVal, x2 = maxVal, mean1 = meanVal, mean2 = meanVal,
            median1 = medianVal, median2 = medianVal
        )
    })
    return(sumDat)
}

.prepareDataHistogram <- function(res, fName) {
    if ("lefser_df" == class(res)[1]) {
        tse <- attr(res, "inputSE")
        sampleData <- as.data.frame(SummarizedExperiment::colData(tse))
        mat <- SummarizedExperiment::assay(tse)
    } else if ("lefser_df_clades" == class(res)[1]) {
        seL <- attr(res, "inputSE")
        sampleData <- as.data.frame(SummarizedExperiment::colData(seL[[1]]))
        matL <- purrr::map(seL, SummarizedExperiment::assay)
        mat <- do.call("rbind", matL)
    } else {
        stop(
            "Need an object of class lefser_df or lefser_df_class",
            call. = FALSE
        )
    }
    classCol <- attr(res, "class_arg")
    subclassCol <- attr(res, "subclass_arg")
    refclass <- attr(res, "lclassf")
    selectCols <- c(classCol, subclassCol)
    # sampleData <- as.data.frame(SummarizedExperiment::colData(tse))
    sampleData <- sampleData[, selectCols, drop = FALSE]
    sampleData <- tibble::rownames_to_column(sampleData, var = "sample")
    mat <- mat[res[["features"]], ]
    mat <- apply(mat, 2, function(x) x / sum(x))
    dat <- mat |> 
        as.data.frame() |>
        tibble::rownames_to_column(var = "features") |>
        tidyr::pivot_longer(
            cols = 2:tidyselect::last_col(),
            names_to = "sample",
            values_to = "abundance"
        ) |>
        dplyr::left_join(sampleData, by = "sample")
    dat[[classCol]] <- forcats::fct_relevel(dat[[classCol]], refclass)

    if (is.null(subclassCol)) {
        dat <- dat |>
            dplyr::filter(.data[["features"]] == fName) |>
            dplyr::arrange(
                .data[[classCol]],
                dplyr::desc(.data[["abundance"]])
            ) |>
            dplyr::mutate(sample = forcats::fct_inorder(.data[["sample"]]))
        colnames(dat)[which(colnames(dat) == classCol)] <- "classCol"
    } else if (!is.null(subclassCol)) {
        dat <- dat |>
            dplyr::filter(.data[["features"]] == fName) |>
            dplyr::arrange(
                .data[[classCol]],
                dplyr::desc(.data[[subclassCol]]),
                dplyr::desc(.data[["abundance"]])
            ) |>
            dplyr::mutate(sample = forcats::fct_inorder(.data[["sample"]]))
        colnames(dat)[which(colnames(dat) == classCol)] <- "classCol"
        colnames(dat)[which(colnames(dat) == subclassCol)] <- "subclassCol"
    }
    return(dat)
}

.isLefser <- function(x) {
    "lefser_df" %in% class(x)
}

utils::globalVariables(c("features", "scores"))

#' Plots results from `lefser` function
#'
#' This function plots the biomarkers found by LEfSe, that are ranked according
#' to their effect sizes and linked to their abundance in each class.
#'
#' @import ggplot2
#' @importFrom dplyr %>% arrange mutate row_number
#' @importFrom utils head tail
#'
#' @param df Data frame produced by \code{lefser}. This data frame contains
#' two columns labeled as `c("features", "scores")`.
#' @param colors Colors corresponding to class 0 and 1.
#' Options: "c" (colorblind), "l" (lefse), "g" (greyscale). Defaults to "c".
#' This argument also accepts a character(2) with two color names.
#' @param trim.names Under the default (`TRUE`), this function extracts the
#' most specific taxonomic rank of organism.
#' @param title A character(1). The title of the plot.
#' @param label.font.size A numeric(1). The font size of the feature labels.
#' The default is `3`.
#'
#' @return
#' Function returns plot of effect size scores produced by \code{lefser}.
#' Positive scores represent the biomarker is more abundant in class '1'.
#' Negative scores represent the biomarker is more abundant in class '0'.
#'
#' @examples
#' example("lefser")
#' lefserPlot(res_class)
#'
#' @export
lefserPlot <- function(df,
                       colors = "c",
                       trim.names = TRUE,
                       title = "",
                       label.font.size = 3) {

    df <- .trunc(df, trim.names)
    classes <- attr(df, "classes")
    colors <- .selectPalette(colors)

    ## Create the `class` column
     if (!is.null(classes)) {
         class <- ifelse(df$scores > 0, tail(classes, 1), head(classes, 1))
         df$class <- factor(class, levels = classes)
    } else {
        class <- ifelse(df$scores > 0, 1, 0)
        df$class <- as.factor(class)
    }

    ## Add the `order` column based on the scores
    ## To make duplicated features behave independently
    df <- df %>%
        arrange(scores) %>%
        mutate(order = row_number())
        # mutate(order = seq_len(nrow(.)))

    plt <-
        ggplot(df, aes(factor(order), scores, width = 0.75)) + # Plot same x-axis values separately
        ylab("LDA SCORE (log 10)") +
        ggtitle(title) +
        theme_void() +
        theme(
            axis.title.y = element_blank(),
            axis.title.x = element_text(size = 11),
            axis.text.y  = element_blank(),
            axis.text.x  = element_text(vjust = 0.7, size = 9)) +
        geom_bar(
            stat = "identity", aes(fill = class), color = "black", size = 0.3) +
        theme(    # Legends
            legend.position = "top",
            legend.title = element_blank(),
            legend.key.height = unit(0.07, 'cm'),
            legend.key.width = unit(0.6, 'cm')) +
        scale_fill_manual(values = colors) +
        geom_text(    # Feature labeling
            aes(y = 0, label = features),
            hjust = ifelse(df$scores < 0, 0, 1),
            nudge_y = ifelse(df$scores < 0, 0.1, -0.1),
            color = "black",
            size = label.font.size) +
        theme(    # Guide lines
            panel.grid.major.x = element_line(
                color = "grey", linewidth = 0.5, linetype = "dotted"),
            panel.grid.minor.x = element_line(
                color = "grey", linewidth = 0.5, linetype = "dotted")) +
        coord_flip()

      return(plt)
}

#' Utility function to calculate relative abundances
#'
#' @description
#' The function calculates the column totals and divides each value within the
#' column by the respective column total.
#'
#' @inheritParams lefser
#'
#' @param se A [SummarizedExperiment][SummarizedExperiment::SummarizedExperiment-class]
#' object with counts
#'
#' @examples
#'
#' se <- SummarizedExperiment(
#'     assays = list(
#'         counts = matrix(
#'             rep(1, 4), ncol = 1, dimnames = list(LETTERS[1:4], "SAMP")
#'         )
#'     )
#' )
#' assay(se)
#' assay(relativeAb(se))
#'
#' @description
#' This function calculates the relative abundance of each feature in the
#' [SummarizedExperiment][SummarizedExperiment::SummarizedExperiment-class]
#' object containing count data, expressed as counts per million (CPM)
#'
#' @returns returns a new
#' [SummarizedExperiment][SummarizedExperiment::SummarizedExperiment-class]
#' object with counts per million calculated and added as a new assay named rel_abs.
#'
#' @export
relativeAb <- function(se, assay = 1L) {
  assay_data <- assay(se, i = assay)
  csums <- colSums(assay_data)
  div <- matrix(rep(csums, each = nrow(assay_data)), ncol = ncol(assay_data))
  res <- assay_data / div * 1e6
  assaylist <- assays(se)
  newalist <- append(
    assaylist, values = S4Vectors::SimpleList(rel_abs = res), after = 0L
  )
  assays(se) <- newalist
  se
}

#' Identify which elements of a string are terminal nodes
#'
#' @param string A character vector of strings to check for terminal nodes
#'
#' @return A logical vector indicating which elements of the string are terminal
#' nodes
#' @description
#' A terminal node in a taxonomy does not have any child nodes. For example, a
#' species is a terminal node if there are no subspecies or strains that belong
#' to that species. This function identifies which elements of a vector are terminal
#' nodes simply by checking whether that element appears as a substring in any other
#' element of the vector.
#'
#' @export
#'
#' @examples
#' # What does it do?
#' data("zeller14")
#' rownames(zeller14)[988:989]
#' get_terminal_nodes(rownames(zeller14)[988:989])
#' # How do I use it to keep only terminal nodes for a lefser analysis?
#' terminal_nodes <- get_terminal_nodes(rownames(zeller14))
#' zeller14sub <- zeller14[terminal_nodes, ]
#' # Then continue with your analysis!
get_terminal_nodes <- function(string) {
  terminal_nodes <- logical(length(string)) # Initialize logical vector
  for (i in seq_along(string)) {
    # Check if the string appears as a substring in any other strings
    if (!any(grepl(string[i], string[-i], fixed = TRUE))) {
      terminal_nodes[i] <- TRUE
    }
  }
  return(terminal_nodes)
}


# Truncate the feature name
.trunc <- function(scores_df, trim.names){
    Names <- gsub("`", "", scores_df[["features"]])
    if (trim.names) {
        listNames <- strsplit(Names, "\\||\\.")
        Names <- vapply(listNames, tail, character(1L), 1L)
    }
    scores_df[["features"]] <- Names
    return(scores_df)
}

## A function for selecting colors for the plots
.selectPalette <- function(x = "colorblind") {
    if (is.character(x) && length(x) == 1) {
        sel <- match.arg(x, choices = c("colorblind", "lefse", "greyscale"))
        message("Using palette: ", sel)
        presetColors <- list(
            colorblind = c("#E57A77", "#7CA1CC"),
            lefse = c("red", "forestgreen"),
            greyscale = c("grey30", "grey60")
        )
        selectedColor <- presetColors[[sel]]
        return(selectedColor)
    } else {
        return(x)
    }
}

#' RowNames to RowData
#' 
#' \code{rowNames2RowData} transforms the taxonomy stored in the row names to 
#' the rowData in a
#' [SummarizedExperiment][SummarizedExperiment::SummarizedExperiment-class].
#' 
#' @param x A
#' [SummarizedExperiment][SummarizedExperiment::SummarizedExperiment-class]
#' with the features taxonomy in the rownames.
#'
#' @return The same
#' [SummarizedExperiment][SummarizedExperiment::SummarizedExperiment-class]
#' with the taxonomy now in the rowData.
#' @export
#'
#' @examples
#'
#' data("zeller14")
#' 
#' ## Keep only "CRC" and "control" (dichotomous variable)
#' z14 <- zeller14[, zeller14$study_condition %in% c("control", "CRC")]
#' 
#' ## Get terminal nodes
#' tn <- get_terminal_nodes(rownames(z14))
#' z14_tn <- z14[tn, ]
#' 
#' ## Normalize to relative abundance (also known as Total Sum Scaling)
#' z14_tn_ra <- relativeAb(z14_tn)
#' 
#' ## Add the taxonomy to the rowData
#' input_se <- rowNames2RowData(z14_tn_ra)
#' 
rowNames2RowData <- function(x) {
    se <- x
    taxonomy <- .getTaxonomyFromPathStr(rownames(se))
    dataFrame <- data.frame(tax = taxonomy) |>
        tidyr::separate(
            col = "tax", into = paste0("col", 1:10), # Number of taxa is usually seven, so 10 should be more than enough.
            sep = "\\|", extra = "merge", fill = "right"
        ) |>
        purrr::discard(~ all(is.na(.x)))
    ## purrr::map_chr ensures that the a single letter is used per column.
    ## Having two or more letters would trigger and error message from map_chr.
    firstLetter <- purrr::map_chr(dataFrame, ~ {
        taxLvl <- stringr::str_extract(.x, "\\w__")
        unique(taxLvl[which(!is.na(taxLvl))])
    })
    rankNames <- dplyr::case_when(
        firstLetter == "k__" ~ "kingdom",
        firstLetter == "p__" ~ "phylum",
        firstLetter == "c__" ~ "class",
        firstLetter == "o__" ~ "order",
        firstLetter == "f__" ~ "family",
        firstLetter == "g__" ~ "genus",
        firstLetter == "s__" ~ "species",
        firstLetter == "t__" ~ "strain",
    )
    colnames(dataFrame) <- rankNames
    dataFrame <- purrr::modify(dataFrame, ~ sub("^\\w__", "", .x))
    DF <- S4Vectors::DataFrame(dataFrame)
    SummarizedExperiment::rowData(se) <- DF
    return(se)
}

checkEnding <- function(resObj, index, column, value) {
    endsWith(resObj[index, column], value)
}

library(lefser)
dataenv <- new.env(parent = emptyenv())
data("zeller14", package = "lefser", envir = dataenv)
zeller14 <- dataenv[["zeller14"]]
zeller142 <- zeller14[, zeller14$study_condition != "adenoma"]
tn <- get_terminal_nodes(rownames(zeller142))
zellersub <- zeller142[tn, ]
zellersubra <- relativeAb(zellersub)

## Set the level for class
colData(zellersubra)$study_condition <- factor(colData(zellersubra)$study_condition,
                                               levels = c("control", "CRC"))

test_that("lefser and lefserPlot work", {
    
    ## This is a fractional tolerance, i.e. 0.001 -> 0.1% fractional difference allowed
    tol <- 0.001 
    
    ## Subsetting a DataFrame with NULL
    set.seed(1234)
    expect_warning(expect_error(lefser(zellersub, classCol = NULL, subclassCol = NULL)))
    withr::with_seed(1,
      results <- lefser(zellersubra, classCol = "study_condition", subclassCol = NULL)
    )
    # TODO: compare results between LEfSe and lefser
    expect_equal(results[1, "scores"], -4.313678, tolerance = tol)
    expect_equal(results[nrow(results), "scores"], 4.099828, tolerance = tol)
    withr::with_seed(1, 
      results2 <- lefser(zellersubra, classCol = "study_condition", subclassCol = "age_category")
    )
    expect_equal(nrow(results2), 15)
    expect_equal(results2$scores,
                 c(-3.795320, -3.700813, -3.521247, -3.154544, -3.126245,
                   -3.082713, -2.962662,  2.141700,  2.286564,  2.431915,
                   2.579108,  2.706471, 2.834329,  2.941230,  3.336170), 
                 tolerance = tol)
    
    # Perform text-based checks only if system locale is en_US.UTF-8
    skip_if_not(identical(Sys.getlocale("LC_COLLATE"), "en_US.UTF-8"))
    expect_equal(colnames(results), c("features", "scores"))
    expect_true(checkEnding(results, 1, "features", "t__GCF_000209875"))
    expect_true(
        checkEnding(results, nrow(results), "features", "s__Subdoligranulum_unclassified")
    )
    expect_equal(colnames(results2), c("features", "scores"))
    expect_true(checkEnding(results2, 1, "features", "t__GCF_000159975"))
    expect_true(checkEnding(
        results2, nrow(results2), "features", "s__Oscillibacter_unclassified"
    ))
    expect_true(all(
        mapply(endsWith, results2$features[14:15], c(
            "t__GCF_000147675",
            "s__Oscillibacter_unclassified"
        ))
    ))
    
    plt <- lefserPlot(results2)
    expect_s3_class(plt, "ggplot")
})

test_that("no significant results behaviors are consistent", {
    expect_message(
        res1 <- lefser(zellersubra, classCol = "study_condition", subclassCol = "age_category", wilcox.threshold = 1e-10),
        "No significant features found."
    )
    expect_message(
        res2 <- lefser(zellersubra, classCol = "study_condition", subclassCol = "age_category", kruskal.threshold = 1e-10),
        "No significant features found."
    )
    expect_message(
        res3 <- lefser(zellersubra, classCol = "study_condition", subclassCol = "age_category", lda.threshold = 1e6),
        "No significant features found."
    )
    expect_equal(nrow(res1), 0L)
    expect_equal(nrow(res2), 0L)
    expect_equal(nrow(res3), 0L)
    expect_equal(colnames(res1), c("features", "scores"))
    expect_equal(colnames(res2), c("features", "scores"))
    expect_equal(colnames(res3), c("features", "scores"))
})

test_that("Relative abundance", {
    expect_warning(lefser(zellersub, classCol = "study_condition"), regexp = "^Convert counts.+")
    expect_equal(assayNames(zellersubra), c("rel_abs", "exprs"))
    expect_equal(colSums(assay(zellersubra, "rel_abs")), rep(1e6, ncol(zellersubra)), check.attributes = FALSE)
    set.seed(1)
    expect_warning(x <- lefser(zeller142, classCol = "study_condition"),
                   "Variables in the input are collinear")
})

data(zeller14)
zeller14 <- zeller14[, zeller14$study_condition != "adenoma"]
tn <- get_terminal_nodes(rownames(zeller14))
zeller14tn <- zeller14[tn,]
zeller14tn_ra <- relativeAb(zeller14tn)

# (1) Using classes only
res_class <- lefser(zeller14tn_ra,
                    classCol = "study_condition")
# (2) Using classes and sub-classes
res_subclass <- lefser(zeller14tn_ra,
                    classCol = "study_condition",
                    subclassCol = "age_category")
plot_class <- lefserPlotFeat(res_class, res_class$features[[1]])
plot_subclass <- lefserPlotFeat(res_subclass, res_subclass$features[[2]])

test_that("lefserPlotFeat works", {
  expect_s3_class(plot_class, "ggplot")
  expect_s3_class(plot_subclass, "ggplot")
})

data("zeller14")
z14 <- zeller14[, zeller14$study_condition != "adenoma"]
tn <- get_terminal_nodes(rownames(z14))
z14tn <- z14[tn, ]
z14tn_ra <- relativeAb(z14tn)

res <- lefser(z14tn_ra, classCol = "study_condition")

z14_input <- rowNames2RowData(z14tn_ra)
resClades <- lefserClades(z14_input, classCol = "study_condition")

test_that("lefserClades works", {
    expect_s3_class(resClades, "lefser_df_clades")
})

test_that("lefserPlotClad works", {
    ggt <- lefserPlotClad(df = resClades)
    expect_s3_class(ggt, "ggtree")
})

library(lefser)

zell <- zeller14[, zeller14$study_condition != "adenoma"]
nr <- nrow(zell)
exprs <- assay(zell, i = 1L)
datas <- data.frame(
    condition = lefser:::.numeric01(zell$study_condition),
    age = lefser:::.numeric01(zell$age_category),
    subjectID = zell$subjectID,
    t(exprs)
)

text <- mapply(function(x, y) {
    paste0(x, "\t", y)
    }, x = names(datas), y = sapply(datas, paste0, collapse = "\t")
)

## file for http://huttenhower.sph.harvard.edu/galaxy/ LEfSe
# writeLines(text, con = file("~/data/lefser/zelle14_150.txt"))
writeLines(text,
    con = file(paste0("~/data/lefser/zelle14_", nr, ".txt")))

## compared to results from lefser
loc <- Sys.getlocale("LC_COLLATE")
Sys.setlocale("LC_COLLATE", "C")
on.exit(Sys.setlocal("LC_COLLATE", loc))

# seed from
# https://github.com/SegataLab/lefse/blob/a4b3140c34b3abe5579a916a20c2a8659c3ac53c/lefse.py#L9
set.seed(1982)
res <- lefser(zell, classCol = "study_condition", subclassCol = "age_category")
lefserPlot(res)

#' Run lefser at different clades
#'
#' \code{lefesrCaldes} Agglomerates the features abundance at different
#' taxonomic ranks using [mia::splitByRanks][mia::splitByRanks]
#' and performs lefser at each rank. The analysis is run at the species,
#' genus, family, order, class, and phylum levels.
#'
#' @param relab A
#' (Tree) [SummarizedExperiment][SummarizedExperiment::SummarizedExperiment-class]
#' with full taxonomy in the rowData @param ... Arguments passed to the
#' \code{lefser} function.
#'
#' @param \ldots Additional arguments passed to `lefser`
#'
#' @return An object of class 'lefser_df_clades', "lefser_df", and 'data.frame'.
#'
#' @details
#'
#' When running \code{lefserClades}, all features with NAs in the rowData will
#' be dropped. This is to avoid creating artificial clades with NAs.
#'
#' @export
#'
#' @examples
#'
#' data("zeller14")
#' z14 <- zeller14[, zeller14$study_condition != "adenoma"]
#' tn <- get_terminal_nodes(rownames(z14))
#' z14tn <- z14[tn, ]
#' z14tn_ra <- relativeAb(z14tn)
#' z14_input <- rowNames2RowData(z14tn_ra)
#'
#' resCl <- lefserClades(relab = z14_input, classCol = "study_condition")
#'
lefserClades <- function(relab, ...) {
    se <- .selectTaxRanks(relab)
    se <- .appendRankLetter(se)
    l <- .dropFeatures(se)
    se <- l[["se"]]
    pathStrings <- l[["pathStrings"]]
    seL <- as.list(mia::splitByRanks(se))
    ## Kingdom would not be informative
    # seL <- seL[!names(seL) %in% "kingdom"]
    msgRanks <- paste(names(seL), collapse = ", ")
    msgRanks <- msgRanks[length(msgRanks):1]
    message(
        "lefser will be run at the ", msgRanks, " level."
    )
    seL <- purrr::imap(seL, function(x, idx) {
        seVar <- x
        row_data <- as.data.frame(SummarizedExperiment::rowData(seVar))
        row_data <- row_data[,1:which(colnames(row_data) == idx), drop = FALSE]
        # row_data <- purrr::discard(row_data, ~ all(is.na(.x)))
        SummarizedExperiment::rowData(seVar) <- S4Vectors::DataFrame(row_data)
        newRowNames <- .rowData2PathStrings(seVar)
        BiocGenerics::rownames(seVar) <- newRowNames
        seVar
    })
    resL <- purrr::imap(seL, function(x, idx, ...) {
        message(
            "\n>>>> Running lefser at the ", idx, " level.",
            " <<<<"
        )
        withCallingHandlers(
            lefser(relab = x,...),
            warning = function(w) {
                if (grepl("relativeAb", w$message)) {
                    invokeRestart("muffleWarning")
                }
            }
        )
    },
    ...
    )
    controlVar <- resL |>
        purrr::map(~ attr(.x, "lclassf")) |>
        unlist(use.names = FALSE) |>
        unique()
    caseVar <- resL |>
        purrr::map(~ attr(.x, "case")) |>
        unlist(use.names = FALSE) |>
        unique()
    classArgVar <- resL |>
        purrr::map(~ attr(.x, "class_arg")) |>
        unlist(use.names = FALSE) |>
        unique()
    subclassArgVar <- resL |>
        purrr::map(~ attr(.x, "subclass_arg")) |>
        unlist(use.names = FALSE) |>
        unique()
    names(resL) <- names(seL)
    res <- dplyr::bind_rows(resL, .id = "Rank") |>
        dplyr::relocate(.data$Rank, .after = tidyselect::last_col())
    class(res) <- c("lefser_df_clades", class(res))
    attr(res, "pathStrings") <- pathStrings
    attr(res, "tree") <- .toTree(pathStrings)
    attr(res, "lclassf") <- controlVar
    attr(res, "case") <- caseVar
    attr(res, "inputSE") <- seL
    attr(res, "class_arg") <- classArgVar
    attr(res, "subclass_arg") <- subclassArgVar
    return(res)
}

## This functions makes sure that only the taxonomy
## is used for the rowData.
## OTU's or other non-typical taxonomic ranks will not be included.
.getTaxonomyFromPathStr <- function(pathStrings) {
    rgx <- "^k__[^|]+\\|p__[^|]+\\|c__[^|]+\\|o__[^|]+\\|f__[^|]+(\\|g__[^|]+)?(\\|s__[^|]+)?(\\|t__[^|]+)?"
    stringr::str_extract(pathStrings, pattern = rgx)
}

## This function selects pathStrings containing only
## taxa that is differentiallty abundant
.selectPathStrings <- function(se, res) {
    pathStrings <- rownames(se)
    index <- res$features |>
        purrr::map(~ which(stringr::str_detect(pathStrings, .x))) |>
        unlist() |>
        unique() |>
        sort()
    pathStrings <- pathStrings[index]
    return(pathStrings)
}

.rowData2PathStrings <- function(se) {
    xDat <- as.data.frame(SummarizedExperiment::rowData(se))
    pathStrings <- tidyr::unite(
        data = xDat, col = "taxonomy", sep = "|",
        1:tidyselect::last_col()
    ) |>
        dplyr::pull(.data$taxonomy)
    pathStrings <- sub("(\\|NA)+$", "", pathStrings)
    return(pathStrings)
}

.selectTaxRanks <- function(x) {
    taxNames <- c(
        "kingdom", "phylum", "class", "order", "family",
        "genus", "species", "strain"
    )
    se <- x
    rowDat <- SummarizedExperiment::rowData(se)
    colnames(rowDat) <- stringr::str_to_lower(colnames(rowDat))
    if ("superkingdom" %in% colnames(rowDat)) {
        message("superkingdom --> kingdom")
        colnames(rowDat)[which(colnames(rowDat) == "superkingdom")] <- "kingdom"
    }
    selectCols <- intersect(taxNames, colnames(rowDat))
    SummarizedExperiment::rowData(se) <- rowDat[, selectCols]
    return(se)
}

.appendRankLetter <- function(x) {
    se <- x
    rowDat <- SummarizedExperiment::rowData(se)
    for (i in seq_along(rowDat)) {
        chr_vct <- rowDat[[i]]
        rankName <- colnames(rowDat)[i]
        rankLetter <- dplyr::case_when(
            rankName == "kingdom" ~ "k__",
            rankName == "phylum" ~ "p__",
            rankName == "class" ~ "c__",
            rankName == "order" ~ "o__",
            rankName == "family" ~ "f__",
            rankName == "genus" ~ "g__",
            rankName == "species" ~ "s__",
            rankName == "strain" ~ "t__",
        )
        # rankLetter <- stringr::str_c(
        #     stringr::str_extract(colnames(rowDat)[i], "^\\w"), "__"
        # )
        rowDat[[i]] <- stringr::str_c(rankLetter, chr_vct)
    }
    SummarizedExperiment::rowData(se) <- rowDat
    return(se)
}

.dropFeatures <- function(x) {
    se <- x
    row_data <- as.data.frame(SummarizedExperiment::rowData(se))
    nrow1 <- nrow(row_data)
    for (i in seq_along(row_data)) {
        sumNA <- sum(is.na(row_data[[i]]))
        if (sumNA > 0) {
            message(
                sumNA, " features don't have ", colnames(row_data)[i],
                " information."
            )
        }
    }
    row_data <- tidyr::drop_na(row_data)
    nrow2 <- nrow(row_data)
    if (nrow1 > nrow2) {
        message(
            "Dropped ", nrow1 - nrow2,
            " features without full taxonomy information."
        )
    }
    se <- se[rownames(row_data),]
    pathStrings <- sort(unique(.rowData2PathStrings(se)))
    l1 <- length(pathStrings)
    tips <- stringr::str_extract(pathStrings, "(|\\w__\\w+)?$")
    dupTips <- tips[which(duplicated(tips))]
    pathStrings <- pathStrings[!tips %in% dupTips]
    l2 <- length(pathStrings)
    if (l1 > l2) {
        message(
            "Dropped ", l1 - l2,
            " features with duplicated tip names."
        )
    }
    list(se = se, pathStrings = pathStrings)
}

## Convert a character vector with pathStrings into a cladogram
## These could come from the rownames of a SummarizedExperiment with
## terminal nodes
.toTree <- function(pathStrs) {
    edgeDF <- pathStrs |>
        purrr::map(.pathString2EdgeList) |>
        dplyr::bind_rows() |>
        dplyr::distinct()
    tipLabels <- stringr::str_extract(pathStrs, "[^|]+$")
    nodeLabels <- unique(edgeDF$from)
    idMap <- 1:(length(tipLabels) + length(nodeLabels))
    names(idMap) <- c(tipLabels, nodeLabels)
    edgeMat <- matrix(
        data = c(idMap[edgeDF$from], idMap[edgeDF$to]),
        ncol = 2
    )
    tr <- list(
        edge = edgeMat,
        tip.label = tipLabels,
        node.label = nodeLabels,
        Nnode = length(nodeLabels),
        Ntip = length(tipLabels)
    )
    class(tr) <- "phylo"
    tr
}

## Helper function for .toTree
## Input is a single path string, e.g., "k__bacteria|p_Fusobacteria..."
.pathString2EdgeList <- function(pathStr) {
    pathStrRoot <- stringr::str_c("ROOT|", pathStr)
    chr_vct <- stringr::str_split(pathStrRoot, "\\|")[[1]]
    data.frame(
        from = chr_vct[1:length(chr_vct)-1],
        to = chr_vct[2:length(chr_vct)]
    )
}

#' LEfSer plot cladogram
#' 
#' \code{lefserPlotClad} plots a cladogram from the results of
#' \code{lefserClades}.
#'
#' @param df An object of class "lefesr_df_clades".
#' @param colors Colors corresponding to class 0 and 1.
#' Options: "c" (colorblind), "l" (lefse), "g" (greyscale).
#' Defaults to "c".
#' This argument also accepts a character(2) with two color names.
#' @param showTipLabels Logical. If TRUE, show tip labels. Default is FALSE.
#' @param showNodeLabels  Label's to be shown in the tree.
#' Options: "p" = phylum, "c" = class, "o" = order,
#' "f" = family, "g" = genus, "s" = species, "t" = strain.
#' It can accept several options, e.g., c("p", "c").
#'
#' @importFrom ggtree %<+%
#'
#' @return A ggtree object.
#' @export
#'
#' @examples
#' data("zeller14")
#' z14 <- zeller14[, zeller14$study_condition != "adenoma"]
#' tn <- get_terminal_nodes(rownames(z14))
#' z14tn <- z14[tn, ]
#' z14tn_ra <- relativeAb(z14tn)
#' z14_input <- rowNames2RowData(z14tn_ra)
#'
#' resCl <- lefserClades(relab = z14_input, classCol = "study_condition")
#' ggt <- lefserPlotClad(df = resCl)
lefserPlotClad <- function(
        df, colors = "c", showTipLabels = FALSE, showNodeLabels = "p"
) {
    inputClass <- class(df)[1]
    if (inputClass != "lefser_df_clades") {
        stop(
            "An object of class 'lefser_df_class' is needed.",
            " Please use the `lefserClades` function.",
            call. = FALSE
        )
    }
    
    df$features <- .extractTips(df$features)
    
    colors <- .selectPalette(colors)
    tree <- attr(df, "tree")
    controlVar <- attr(df, "lclassf")
    caseVar <- attr(df, "case")
    
    res <- df |>
        dplyr::mutate(
            sample = dplyr::case_when(
                ## This assumes positive values always mean enriched in
                ## the case condition.
                .data[["scores"]] > 0 ~ .env[["caseVar"]],
                TRUE ~ .env[["controlVar"]]
            )
        ) |>
        dplyr::mutate(abs = abs(.data[["scores"]])) |>
        as.data.frame()
    
    labels <- c(tree$tip.label, tree$node.label)
    res$node <- match(res$features, labels)
    dat <- dplyr::relocate(res, .data$node)
    
    internalNodes <- ape::Ntip(tree) + 1:ape::Nnode(tree)
    
    collapseThem <- purrr::map_int(internalNodes, ~ {
        chNods <- treeio::offspring(.data = tree, .node = .x, type = "tips")
        if (any(chNods %in% dat$node)) {
            return(NA)
        } else {
            return(.x)
        }
    }) |>
        purrr::discard(is.na)
    
    nodLab <- match.arg(
        arg = showNodeLabels,
        choices = c("p", "c", "o", "f", "g", "s", "t"),
        several.ok = TRUE
    )
    nodLabRgx <- paste0("[", paste0(nodLab, collapse = ""), "]__")
    treeData <- dat |>
        dplyr::mutate(abs = round(.data$abs, 1)) |> 
        dplyr::mutate(
            showNodeLabs = dplyr::case_when(
                grepl(nodLabRgx, features) ~ features,
                TRUE ~ NA
            )
        )
    minval <- floor(min(treeData$abs, na.rm = TRUE))
    maxval <- floor(max(treeData$abs, na.rm = TRUE))
    
    gt <- ggtree::ggtree(
        tree, layout = "circular",  branch.length = "none", size = 0.2
    ) %<+% treeData
    
    gt <- gt +
        ggtree::geom_tippoint(
            mapping = ggtree::aes(fill = sample, size = abs), shape = 21,
            na.rm=TRUE
        ) +
        ggtree::geom_nodepoint(
            mapping = ggtree::aes(fill = sample, size = abs), shape = 21,
            na.rm = TRUE
        )
    
    if (showTipLabels) {
        gt <- gt + 
            ggtree::geom_tiplab(
                mapping = ggtree::aes(label = features), size = 2,
                geom = "text", na.rm=TRUE
            )
    }
    
    gt2 <- gt +
        ggrepel::geom_label_repel(
            mapping = ggtree::aes(label = .data$showNodeLabs),
            na.rm = TRUE
        ) +
        ggtree::scale_fill_manual(
            values = colors, breaks = c(controlVar, caseVar),
            name = "Sample", na.value = NA
        ) +
        ggplot2::scale_size(
            name = "Absolute\nLDA score",
            limits = c(minval, maxval),
            # range = range(seq_along(seq(minval, maxval, 1))),
            range = c(minval, maxval),
            breaks = seq(minval, maxval, 1)
            
        ) +
        ggtree::theme(legend.position = "right")
    
    for (i in collapseThem) {
        gt2 <- withCallingHandlers(
            ggtree::collapse(gt2, node = i),
            warning = function(w) {
                if (grepl("collapse", w$message)) {
                    invokeRestart("muffleWarning")
                }
            }
        )
    }
    return(gt2)
}

## This function extracts only the last element of the taxonomy
.extractTips <- function(pathStrs) {
    stringr::str_extract(pathStrs, "[^|]+$")
}

#' Example dataset for lefser
#'
#' The ZellerG_2014 dataset contains microbiome count data for CRC patients and
#' controls. It was for curatedMetagenomicData using the script in the package
#' directory "data-raw".
#'
#' @usage data("zeller14")
#' @format A [SummarizedExperiment][SummarizedExperiment::SummarizedExperiment-class]
#' with 1585 features, 199 samples
#' \describe{
#'   \item{study_condition}{adenoma, control, CRC}
#'   \item{age_category}{adult, senior}
#' }
#' @source \url{https://pubmed.ncbi.nlm.nih.gov/25432777/}
"zeller14"

suppressPackageStartupMessages({
    library(MicrobiomeBenchmarkData)
    library(lefser)
})

## Example 1 - Strains
## Clades will be summarized at the species level and above
data("zeller14")
z14 <- zeller14[, zeller14$study_condition %in% c("control", "CRC")]
tn <- get_terminal_nodes(rownames(z14))
z14_tn <- z14[tn, ]
z14_tn_ra <- relativeAb(z14_tn)
z14_input <- rowNames2RowData(z14_tn_ra)

resCl <- lefserClades(relab = z14_input, classCol = "study_condition")
(ggt <- lefserPlotClad(resCl))

## Example 2 - OTUs
## Clades will be summarized at the genus level and above
tse <- getBenchmarkData("HMP_2012_16S_gingival_V35", dryrun = FALSE)[[1]]
tse_ra <- relativeAb(tse)
res_tse_cld <- lefserClades(relab = tse_ra, classCol = "body_subsite")

(ggt2 <- lefserPlotClad(res_tse_cld))

sessioninfo::session_info()

## code to prepare example dataset goes here
## can change dataset by replacing ZellerG_2014 with any other dataset

library(curatedMetagenomicData)
library(SummarizedExperiment)
exampledata <-
  curatedMetagenomicData("ZellerG_2014.metaphlan_bugs_list.stool",
                         counts = TRUE,
                         dryrun = FALSE)[[1]]
zeller14 <- as(exampledata, "SummarizedExperiment")
colData(zeller14) <- colData(zeller14)[, c(4,8)]
usethis::use_data(zeller14, overwrite = TRUE)

suppressMessages(library(lefser))
data(zeller14)
zeller14 <- zeller14[, zeller14$study_condition != "adenoma"]
tn <- get_terminal_nodes(rownames(zeller14))
zeller14tn <- zeller14[tn,]
zeller14tn_ra <- relativeAb(zeller14tn)

resClass <- lefser(
    zeller14tn_ra, classCol = "study_condition"
)
resSubClass <- lefser(
    zeller14tn_ra, classCol = "study_condition", subclassCol = "age_category"
)
## Plot one feature
plotClass <- lefserPlotFeat(resClass, resClass$features[[1]], colors = "c")
plotClass
plotSubClass <- lefserPlotFeat(resSubClass, resSubClass$features[[1]], colors = "c")
plotSubClass
sessioninfo::session_info()

---
title: "lefser: a metagenomic biomarker discovery tool"
author: Asya Khleborodova, Sehyun Oh, Ludwig Geistlinger, and Levi Waldron
date: "`r Sys.Date()`"
output:
  BiocStyle::html_document:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Quickstart}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(comment = "#>", 
                      fig.align = "center",
                      collapse = FALSE, 
                      message = FALSE, 
                      warning = FALSE, 
                      eval = TRUE)
```

# Overview
## Background
*lefser* is the R implementation of the Linear discriminant analysis (LDA) 
Effect Size ([LEfSe][]), a Python package for metagenomic biomarker discovery 
and explanation. ([Huttenhower et al. 2011][]).

The original software utilizes standard statistical significance tests along 
with supplementary tests that incorporate biological consistency and the 
relevance of effects to identity the features (e.g., organisms, clades, OTU,
genes, or functions) that are most likely to account for differences between
the two sample classes of interest. While *LEfSe* is widely used and available
in different platform such as Galaxy UI and Conda, there is no convenient way
to incorporate it in R-based workflows. Thus, we re-implement *LEfSe* as
an R/Bioconductor package, *lefser*. Following the *LEfSe*‘s algorithm including
Kruskal-Wallis test, Wilcoxon-Rank Sum test, and Linear Discriminant Analysis,
with some modifications, *lefser* successfully reproduces and improves the
original statistical method and the associated plotting functionality.

[LEfSe]: https://huttenhower.sph.harvard.edu/galaxy/
[Huttenhower et al. 2011]: https://www.ncbi.nlm.nih.gov/pmc/articles/PMC3218848/


## Install and load pacakge
```{r eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("lefser")
```

```{r}
library(lefser)
```

## Citing *lefser*

Your citations are crucial in keeping our software free and open source. To
cite our package, please use this publication at the link [here][1].

[1]: https://doi.org/10.1093/bioinformatics/btae707

# Analysis example
## Prepare input

*lefser* package include the demo dataset, `zeller14`, which is the 
microbiome data from colorectal cancer (CRC) patients and controls 
([Zeller et al. 2014][]). 

In this vignette, we excluded the 'adenoma' condition and used control/CRC 
as the main classes and age category as sub-classes (adult vs. senior) with 
different numbers of samples: control-adult (n = 46), control-senior (n = 20), 
CRC-adult (n = 45), and CRC-senior (n = 46). 

[Zeller et al. 2014]: https://www.embopress.org/doi/full/10.15252/msb.20145645

```{r}
data(zeller14)
zeller14 <- zeller14[, zeller14$study_condition != "adenoma"]
```

The class and subclass information is stored in the `colData` slot under the
`study_condition` and `age_category` columns, respectively. 

```{r}
## Contingency table
table(zeller14$age_category, zeller14$study_condition)
```

If you try to run `lefser` directly on the 'zeller14' data, you will get the
following warning messages
```{r eval=FALSE}
lefser(zeller14, classCol = "study_condition", subclassCol = "age_category")
```

```
Warning messages:
1: In lefser(zeller14, classCol = "study_condition", subclassCol = "age_category") :
  Convert counts to relative abundances with 'relativeAb()'
2: In lda.default(x, classing, ...) : variables are collinear
```

### Terminal node
When working with taxonomic data, including both terminal and non-terminal 
nodes in the analysis can lead to collinearity problems. Non-terminal nodes 
(e.g., genus) are often linearly dependent on their corresponding terminal 
nodes (e.g., species) since the species-level information is essentially a 
subset or more specific representation of the genus-level information. This
collinearity can violate the assumptions of certain statistical methods, such
as linear discriminant analysis (LDA), and can lead to unstable or unreliable 
results. By using only terminal nodes, you can effectively eliminate this 
collinearity issue, ensuring that your analysis is not affected by linearly 
dependent or highly correlated variables. Additionally, you can benefit of 
avoiding redundancy, increasing specificity, simplifying data, and reducing
ambiguity, using only terminal nodes.

You can select only the terminal node using `get_terminal_nodes` function. 

```{r}
tn <- get_terminal_nodes(rownames(zeller14))
zeller14tn <- zeller14[tn,]
```


### Relative abundance
First warning message informs you that `lefser` requires relative abundance of
features. You can use `relativeAb` function to reformat your input. 

```{r}
zeller14tn_ra <- relativeAb(zeller14tn)
```


## Run `lefser`
The `lefser` function returns a `data.frame` with two columns - the names of 
selected features (the `features` column) and their effect size (the `scores` 
column).

There is a random number generation step in the `lefser` algorithm to ensure 
that more than half of the values for each features are unique. In most cases, 
inputs are sparse, so in practice, this step is handling 0s. So to reproduce
the identical result, you should set the seed before running `lefser`.

```{r}
set.seed(1234)
res <- lefser(zeller14tn_ra, # relative abundance only with terminal nodes
              classCol = "study_condition",
              subclassCol = "age_category")
head(res)
```


## Visualization using `lefserPlot`
```{r}
lefserPlot(res)
```


# Benchmarking againt other tools
The codes for benchmarking *lefser* against *LEfSe* and the other R
implementation of *LEfSe* is available [here][].

[here]: https://github.com/shbrief/lefserBenchmarking



# Interoperating with phyloseq
When using `phyloseq` objects, we recommend to extract the data and create a
`SummarizedExperiment` object as follows:

```{r warning=FALSE}
library(phyloseq)
library(SummarizedExperiment)

## Load phyloseq object
fp <- system.file("extdata", 
                  "study_1457_split_library_seqs_and_mapping.zip",
                  package = "phyloseq")
kostic <- microbio_me_qiime(fp)

## Split data tables
counts <- unclass(otu_table(kostic))
coldata <- as(sample_data(kostic), "data.frame")

## Create a SummarizedExperiment object
SummarizedExperiment(assays = list(counts = counts), colData = coldata)
```

You may also consider using `makeTreeSummarizedExperimentFromPhyloseq` from 
the `mia` package.
```{r}
mia::makeTreeSummarizedExperimentFromPhyloseq(kostic)
```




# Session Info

<details>
```{r}
sessionInfo()
```
</details>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{zeller14}
\alias{zeller14}
\title{Example dataset for lefser}
\format{
A \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
with 1585 features, 199 samples
\describe{
\item{study_condition}{adenoma, control, CRC}
\item{age_category}{adult, senior}
}
}
\source{
\url{https://pubmed.ncbi.nlm.nih.gov/25432777/}
}
\usage{
data("zeller14")
}
\description{
The ZellerG_2014 dataset contains microbiome count data for CRC patients and
controls. It was for curatedMetagenomicData using the script in the package
directory "data-raw".
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{relativeAb}
\alias{relativeAb}
\title{Utility function to calculate relative abundances}
\usage{
relativeAb(se, assay = 1L)
}
\arguments{
\item{se}{A \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object with counts}

\item{assay}{The i-th assay matrix in the
\code{\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}}
('relab'; #' default 1).}
}
\value{
returns a new
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object with counts per million calculated and added as a new assay named rel_abs.
}
\description{
The function calculates the column totals and divides each value within the
column by the respective column total.

This function calculates the relative abundance of each feature in the
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
object containing count data, expressed as counts per million (CPM)
}
\examples{

se <- SummarizedExperiment(
    assays = list(
        counts = matrix(
            rep(1, 4), ncol = 1, dimnames = list(LETTERS[1:4], "SAMP")
        )
    )
)
assay(se)
assay(relativeAb(se))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lefserPlot.R
\name{lefserPlot}
\alias{lefserPlot}
\title{Plots results from \code{lefser} function}
\usage{
lefserPlot(
  df,
  colors = "c",
  trim.names = TRUE,
  title = "",
  label.font.size = 3
)
}
\arguments{
\item{df}{Data frame produced by \code{lefser}. This data frame contains
two columns labeled as \code{c("features", "scores")}.}

\item{colors}{Colors corresponding to class 0 and 1.
Options: "c" (colorblind), "l" (lefse), "g" (greyscale). Defaults to "c".
This argument also accepts a character(2) with two color names.}

\item{trim.names}{Under the default (\code{TRUE}), this function extracts the
most specific taxonomic rank of organism.}

\item{title}{A character(1). The title of the plot.}

\item{label.font.size}{A numeric(1). The font size of the feature labels.
The default is \code{3}.}
}
\value{
Function returns plot of effect size scores produced by \code{lefser}.
Positive scores represent the biomarker is more abundant in class '1'.
Negative scores represent the biomarker is more abundant in class '0'.
}
\description{
This function plots the biomarkers found by LEfSe, that are ranked according
to their effect sizes and linked to their abundance in each class.
}
\examples{
example("lefser")
lefserPlot(res_class)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_terminal_nodes}
\alias{get_terminal_nodes}
\title{Identify which elements of a string are terminal nodes}
\usage{
get_terminal_nodes(string)
}
\arguments{
\item{string}{A character vector of strings to check for terminal nodes}
}
\value{
A logical vector indicating which elements of the string are terminal
nodes
}
\description{
A terminal node in a taxonomy does not have any child nodes. For example, a
species is a terminal node if there are no subspecies or strains that belong
to that species. This function identifies which elements of a vector are terminal
nodes simply by checking whether that element appears as a substring in any other
element of the vector.
}
\examples{
# What does it do?
data("zeller14")
rownames(zeller14)[988:989]
get_terminal_nodes(rownames(zeller14)[988:989])
# How do I use it to keep only terminal nodes for a lefser analysis?
terminal_nodes <- get_terminal_nodes(rownames(zeller14))
zeller14sub <- zeller14[terminal_nodes, ]
# Then continue with your analysis!
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lefserClades.R
\name{lefserClades}
\alias{lefserClades}
\title{Run lefser at different clades}
\usage{
lefserClades(relab, ...)
}
\arguments{
\item{relab}{A
(Tree) \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
with full taxonomy in the rowData @param ... Arguments passed to the
\code{lefser} function.}

\item{\ldots}{Additional arguments passed to \code{lefser}}
}
\value{
An object of class 'lefser_df_clades', "lefser_df", and 'data.frame'.
}
\description{
\code{lefesrCaldes} Agglomerates the features abundance at different
taxonomic ranks using \link[mia:agglomerate-methods]{mia::splitByRanks}
and performs lefser at each rank. The analysis is run at the species,
genus, family, order, class, and phylum levels.
}
\details{
When running \code{lefserClades}, all features with NAs in the rowData will
be dropped. This is to avoid creating artificial clades with NAs.
}
\examples{

data("zeller14")
z14 <- zeller14[, zeller14$study_condition != "adenoma"]
tn <- get_terminal_nodes(rownames(z14))
z14tn <- z14[tn, ]
z14tn_ra <- relativeAb(z14tn)
z14_input <- rowNames2RowData(z14tn_ra)

resCl <- lefserClades(relab = z14_input, classCol = "study_condition")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rowNames2RowData}
\alias{rowNames2RowData}
\title{RowNames to RowData}
\usage{
rowNames2RowData(x)
}
\arguments{
\item{x}{A
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
with the features taxonomy in the rownames.}
}
\value{
The same
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}
with the taxonomy now in the rowData.
}
\description{
\code{rowNames2RowData} transforms the taxonomy stored in the row names to
the rowData in a
\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}.
}
\examples{

data("zeller14")

## Keep only "CRC" and "control" (dichotomous variable)
z14 <- zeller14[, zeller14$study_condition \%in\% c("control", "CRC")]

## Get terminal nodes
tn <- get_terminal_nodes(rownames(z14))
z14_tn <- z14[tn, ]

## Normalize to relative abundance (also known as Total Sum Scaling)
z14_tn_ra <- relativeAb(z14_tn)

## Add the taxonomy to the rowData
input_se <- rowNames2RowData(z14_tn_ra)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lefserPlotFeat.R
\name{lefserPlotFeat}
\alias{lefserPlotFeat}
\title{Plot Feature}
\usage{
lefserPlotFeat(res, fName, colors = "colorblind")
}
\arguments{
\item{res}{An object of class lefser_df,
output of the \code{lefser} function.}

\item{fName}{A character string. The name of a feature in the lefser_df
object.}

\item{colors}{Colors corresponding to class 0 and 1.
Options: "c" (colorblind), "l" (lefse), "g" (greyscale). Defaults to "c".
This argument also accepts a character(2) with two color names.}
}
\value{
A ggplot object.
}
\description{
\code{lefserPlotFeat} plots the abundance data of a DA feature across all
samples.
}
\details{
The solid lines represent the mean by class or by class+subclass
(if the subclass variable is present).
The dashed lines represent the median by class or by class+subclass
(if the subclass variable is present).
}
\examples{

data(zeller14)
zeller14 <- zeller14[, zeller14$study_condition != "adenoma"]
tn <- get_terminal_nodes(rownames(zeller14))
zeller14tn <- zeller14[tn,]
zeller14tn_ra <- relativeAb(zeller14tn)

# (1) Using classes only
res_class <- lefser(zeller14tn_ra,
                    classCol = "study_condition")
# (2) Using classes and sub-classes
res_subclass <- lefser(zeller14tn_ra,
                    classCol = "study_condition",
                    subclassCol = "age_category")
plot_class <- lefserPlotFeat(res_class, res_class$features[[1]])
plot_subclass <- lefserPlotFeat(res_subclass, res_subclass$features[[2]])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lefserPlotClad.R
\name{lefserPlotClad}
\alias{lefserPlotClad}
\title{LEfSer plot cladogram}
\usage{
lefserPlotClad(df, colors = "c", showTipLabels = FALSE, showNodeLabels = "p")
}
\arguments{
\item{df}{An object of class "lefesr_df_clades".}

\item{colors}{Colors corresponding to class 0 and 1.
Options: "c" (colorblind), "l" (lefse), "g" (greyscale).
Defaults to "c".
This argument also accepts a character(2) with two color names.}

\item{showTipLabels}{Logical. If TRUE, show tip labels. Default is FALSE.}

\item{showNodeLabels}{Label's to be shown in the tree.
Options: "p" = phylum, "c" = class, "o" = order,
"f" = family, "g" = genus, "s" = species, "t" = strain.
It can accept several options, e.g., c("p", "c").}
}
\value{
A ggtree object.
}
\description{
\code{lefserPlotClad} plots a cladogram from the results of
\code{lefserClades}.
}
\examples{
data("zeller14")
z14 <- zeller14[, zeller14$study_condition != "adenoma"]
tn <- get_terminal_nodes(rownames(z14))
z14tn <- z14[tn, ]
z14tn_ra <- relativeAb(z14tn)
z14_input <- rowNames2RowData(z14tn_ra)

resCl <- lefserClades(relab = z14_input, classCol = "study_condition")
ggt <- lefserPlotClad(df = resCl)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lefser.R
\name{lefser}
\alias{lefser}
\title{R implementation of the LEfSe method}
\usage{
lefser(
  relab,
  kruskal.threshold = 0.05,
  wilcox.threshold = 0.05,
  lda.threshold = 2,
  classCol = "CLASS",
  subclassCol = NULL,
  assay = 1L,
  trim.names = FALSE,
  checkAbundances = TRUE,
  method = "none",
  ...
)
}
\arguments{
\item{relab}{A \link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment-class}
with relative abundances in the assay}

\item{kruskal.threshold}{numeric(1) The p-value for the Kruskal-Wallis Rank
Sum Test (default 0.05). If multiple hypothesis testing is performed, this
threshold is applied to corrected p-values.}

\item{wilcox.threshold}{numeric(1) The p-value for the Wilcoxon Rank-Sum Test
when 'subclassCol' is present (default 0.05). If multiple hypothesis testing is
performed, this threshold is applied to corrected p-values.}

\item{lda.threshold}{numeric(1) The effect size threshold (default 2.0).}

\item{classCol}{character(1) Column name in \code{colData(relab)} indicating
class, usually a factor with two levels (e.g., \code{c("cases", "controls")};
default "CLASS").}

\item{subclassCol}{character(1) Optional column name in \code{colData(relab)}
indicating the subclasses, usually a factor with two levels (e.g.,
\code{c("adult", "senior")}; default NULL), but can be more than two levels.}

\item{assay}{The i-th assay matrix in the
\code{\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}}
('relab'; #' default 1).}

\item{trim.names}{Default is \code{FALSE}. If \code{TRUE}, this function extracts
the most specific taxonomic rank of organism.}

\item{checkAbundances}{\code{logical(1)} Whether to check if the assay data in the
\code{relab} input are relative abundances or counts. If counts are found, a
warning will be emitted (default \code{TRUE}).}

\item{method}{Default is "none" as in the original LEfSe implementation.
Character string of length one, passed on to \link[stats]{p.adjust} to set
option for multiple testing. For multiple pairwise comparisons, each comparison
is adjusted separately. Options are "holm", "hochberg", "hommel",
"bonferroni", "BH", "BY", "fdr" (synonym for "BH"), and "none".}

\item{\ldots}{Additional inputs to lower level functions (not used).}
}
\value{
The function returns a \code{data.frame} with two columns, which are
names of features and their LDA scores.
}
\description{
Perform a LEfSe analysis: the function carries out differential analysis
between two sample classes for multiple features and uses linear discriminant analysis
to establish their effect sizes. Subclass information for each class can be incorporated
into the analysis (see examples). Features with large differences between two sample classes
are identified as biomarkers.
}
\details{
The LEfSe method expects relative abundances in the \code{expr} input. A warning
will be emitted if the column sums do not result in 1. Use the \code{relativeAb}
helper function to convert the data in the
\code{\link[SummarizedExperiment:SummarizedExperiment-class]{SummarizedExperiment}}
to relative abundances. The \code{checkAbundances} argument enables checking the
data for presence of relative abundances and can be turned off by setting
the argument to \code{FALSE}.
}
\examples{

    data(zeller14)
    zeller14 <- zeller14[, zeller14$study_condition != "adenoma"]
    tn <- get_terminal_nodes(rownames(zeller14))
    zeller14tn <- zeller14[tn,]
    zeller14tn_ra <- relativeAb(zeller14tn)

    # (1) Using classes only
    res_class <- lefser(zeller14tn_ra,
                        classCol = "study_condition")
    # (2) Using classes and sub-classes
    res_subclass <- lefser(zeller14tn_ra,
                        classCol = "study_condition",
                        subclassCol = "age_category")

}
